package handles

import (
	"fmt"
	"github.com/gorilla/mux"
	"github.com/louisevanderlith/kong"
	"github.com/rs/cors"
	"net/http"
	"strings"
)

func SetupRoutes(host, authUrl, scrt, smtpUser, smtpPass, smtpHost string, smtpPort int) http.Handler {
	r := mux.NewRouter()

	/*
	   "Name": "comms.messages.view",

	   "Name": "comms.messages.create",
	   "Name": "comms.messages.search",
	*/

	view := kong.ResourceMiddleware("comms.messages.view", scrt, authUrl, ViewMessage)
	r.HandleFunc("/message/{key:[0-9]+\\x60[0-9]+}", view).Methods(http.MethodGet)

	create := kong.ResourceMiddleware("comms.messages.create", scrt, authUrl, CreateMessage(smtpUser, smtpPass, smtpHost, smtpPort))
	r.HandleFunc("/message", create).Methods(http.MethodPost)

	search := kong.ResourceMiddleware("comms.messages.search", scrt, authUrl, SearchMessages)
	r.HandleFunc("/message/{pagesize:[A-Z][0-9]+}", search).Methods(http.MethodGet)
	r.HandleFunc("/message/{pagesize:[A-Z][0-9]+}/{hash:[a-zA-Z0-9]+={0,2}}", search).Methods(http.MethodGet)

	corsOpts := cors.New(cors.Options{
		AllowedOrigins: []string{fmt.Sprintf("https://*%s", strings.TrimSuffix(host, "/"))}, //you service is available and allowed for this base url
		AllowedMethods: []string{
			http.MethodGet,
			http.MethodPost,
			http.MethodOptions,
			http.MethodHead,
		},
		AllowCredentials: true,
		AllowedHeaders: []string{
			"*", //or you can your header key values which you are using in your application
		},
	})

	return corsOpts.Handler(r)
}
